/*
 *	project:	bmp2pdb (version 2.1)
 *	file:		bmp2pdb_2.c
 *	copy:		Collin R. Mulliner <palm@mulliner.org>
 *	www:		http://www.mulliner.org/palm/
 *	date:		May 31. 2001
 *
 *	! this belongs to the MyPhone package !
 */


#include "bmp2pdb.h"

// protos
int check_bmp(FILE *fp);
int write_pdb_header(FILE *fp, char *db_name, int mode);
int write_rec_header(FILE *fp, unsigned int current_pos);

int main(int argc, char **argv)
{
	FILE *in, *out;
	int error;
	unsigned int rec_ptrs = 0;
	unsigned char *buffer;
	int err_out = 1;
	int mode = 0;	// bmp2pdb as default
	int i;	

	
	// to work with MSWindows
	for (i = 0; i < strlen(argv[0]); i++) { argv[0][i] = tolower(argv[0][i]); }
	
	// mode ?
	if (strstr(argv[0], "2pdb_silent") != NULL) err_out = 0;
	if (strstr(argv[0], "midi2pdb") != NULL) mode = 1;
	
	if (argc == 4) {
		if ((in = fopen(argv[1], "rb")) == NULL) { if (err_out) { printf(FILE_ERR_TXT); } return(1); }
		if (mode == 0 && (error = check_bmp(in)) != 0) { 
			if (err_out) { 
				switch (error) {
				case IMAGE_SIZE_ERROR:
					printf(SIZE_ERR_TXT);
					break;
				case IMAGE_COMPRESS_ERROR:
					printf(COMPRESS_ERR_TXT);
					break;
				case IMAGE_DPI_ERROR:
					printf(DPI_ERR_TXT);
					break;
				case IMAGE_COLOR_ERROR:
					printf(COLOR_ERR_TXT);
					break;
				}				
			}
			return(error); 
		}
		if ((out = fopen(argv[2], "wb")) == NULL) { if (err_out) { printf(FILE_ERR_TXT); } return(1); }
		
		write_pdb_header(out, argv[3], mode);
		rec_ptrs = sizeof(pdb_db_header_type) + (sizeof(rec_header_type) * 1);
		write_rec_header(out, rec_ptrs);
		
		buffer = (unsigned char *) malloc(10 * 1024);	// 10K
		fread(buffer, 10 * 1024, 1, in);
		fwrite(buffer, ftell(in), 1, out);
		free(buffer);
		
		fclose(in);
		fclose(out);
	}
	else {
		if (err_out) {
			switch (mode) {
			case 0:	printf(BMP_HELP);
				break;
			case 1:	printf(MIDI_HELP);
				break;
			}
		}
	}	
	
	return(0);
}

/*
 *	check if the bmp image file is valid !
 */
int check_bmp(FILE *fp)
{
	bmfh_type bmp_file_header;
	bmih_type bmp_image_header;
	

	if (fread(&bmp_file_header, sizeof(bmfh_type), 1, fp) == 1) {
		if (fread(&bmp_image_header, sizeof(bmih_type), 1, fp) == 1) {
		} else { return (IMAGE_FILE_ERROR); }
	} else { return(IMAGE_FILE_ERROR); }

	if (bmp_image_header.biWidth > MAX_IMAGE_WIDTH || bmp_image_header.biHeight > MAX_IMAGE_HEIGHT) return(IMAGE_SIZE_ERROR);
	if (bmp_image_header.biCompression != 0) return(IMAGE_COMPRESS_ERROR);
	if (bmp_image_header.biBitCount != 1 && bmp_image_header.biBitCount != 4 && bmp_image_header.biBitCount != 8) return(IMAGE_COLOR_ERROR);

// we don't actually need this !
//	if (bmp_image_header.biXPelsPerMeter < DPI_RANGE_MIN || bmp_image_header.biYPelsPerMeter < DPI_RANGE_MIN ||
//		bmp_image_header.biXPelsPerMeter > DPI_RANGE_MAX || bmp_image_header.biYPelsPerMeter > DPI_RANGE_MAX) return(IMAGE_DPI_ERROR);

	// rewind fp so we have to original fp
	rewind(fp);
	
	return(0);
}

/*
 *	write pdb (database) header
 */
int write_pdb_header(FILE *fp, char *db_name, int mode)
{
	pdb_db_header_type pdb_file;
	
	
	memset(&pdb_file, 0x00, sizeof(pdb_db_header_type));
	strncpy(pdb_file.db_name, db_name, 32);
	
	// set creation/modification/backup time to the current time (and date)
	pdb_file.creationTime = time(NULL) + PALMOS_TIME_OFFSET;
	pdb_file.modificationTime = time(NULL) + PALMOS_TIME_OFFSET;
	pdb_file.backupTime = time(NULL) + PALMOS_TIME_OFFSET;
	
	// UGLY !!!
	if (mode == 0) pdb_file.typeID = 'L52S';
	else pdb_file.typeID = 'DIMS';
	pdb_file.creatorID = '6MRC';
	
	pdb_file.numRecords = 1;
	SWAP2(pdb_file.numRecords);
	
	fwrite(&pdb_file, sizeof(pdb_db_header_type), 1, fp);
	
	return(0);
}

/*
 *	write the record header
 */
int write_rec_header(FILE *fp, unsigned int current_pos)
{
	rec_header_type rec_header;		
		
	
	memset(&rec_header, 0x00, sizeof(rec_header_type));
	
	rec_header.offset = current_pos;
	SWAP4(rec_header.offset);
	
	rec_header.flags = 0x00;
	rec_header.uniqueID[0] = 'C';
	rec_header.uniqueID[1] = 'R';
	rec_header.uniqueID[2] = 'M';
	
	fwrite(&rec_header, sizeof(rec_header_type), 1, fp);
	
	return(0);
}
